; Author:      Roy Klein Gebbinck (www.b-k-g.nl)
; Version:     20190414B: Only entities created during the _Mtext and _Field commands are now processed.
;                         The application can again be used during a _RefEdit session.
;              20190414 : Two checks were added to avoid issues with _RefEdit.
;                         The application now does not work during a _RefEdit session.
;              20190413 : First version.
; Created for: https://forum.bricsys.com/discussion/32684/mtext-mask-defaults

; Global variables (user settings):
(setq *BKG_AutoMask_color* nil)         ; Fill color. Examples:
                                        ; (setq *BKG_AutoMask_color* nil)           ; Use drawing background.
                                        ; (setq *BKG_AutoMask_color* 1)             ; Use ACI color.
                                        ; (setq *BKG_AutoMask_color* '(100 150 200) ; Use true color (RGB values).
(setq *BKG_AutoMask_offsetFactor* 1.5)  ; Border offset factor.

; Other global variables:
; *BKG_AutoMask_cmdAndLspCount*
; *BKG_AutoMask_toDoLst*

; Library function
(defun KGA_Sys_Color_Rgb_To_Aci (rgbLst / clr)
  (setq clr
    (cond
      ((vlax-create-object "AutoCAD.AcCmColor")) ; Works from BC V14.
      ((vla-get-truecolor (vla-get-activelayer (vla-get-activedocument (vlax-get-acad-object)))))
    )
  )
  (apply 'vla-setrgb (cons clr rgbLst)) ; Also changes the color method.
  (vla-get-colorindex clr)
)

; Library function
(defun KGA_Sys_Color_Rgb_To_Ole (rgbLst)
  (+
    (lsh (car rgbLst) 16)
    (lsh (cadr rgbLst) 8)
    (caddr rgbLst)
  )
)

; 20190413
; (BKG_AutoMask_AddMask (car (entsel)) 1 5)
(defun BKG_AutoMask_AddMask (enm col fac) ; Col: aci color, RGB list or nil (for drawing background).
  (cond
    ((not col)
      (entmod
        (append
          (entget enm)
          (list
            '(90 . 3)
            '(63 . 257)   ; 257 is the default value. But if a mask was ever attached to the entity in the
                          ; current session, the previous gc63 value (stored where?) would be the default.
            (cons 45 fac)
          )
        )
      )
    )
    ((listp col)
      (entmod
        (append
          (entget enm)
          (list
            '(90 . 1)
            (cons 63 (KGA_Sys_Color_Rgb_To_Aci col))  ; Required!
            (cons 421 (KGA_Sys_Color_Rgb_To_Ole col))
            (cons 45 fac)
          )
        )
      )
    )
    (T
      (entmod
        (append
          (entget enm)
          (list
            '(90 . 1)
            (cons 63 col)
            (cons 45 fac)
          )
        )
      )
    )
  )
)

; 20190414B: Revised. Only entities appended during the _Mtext and _Field commands are processed.
; 20190414 : Added a '_RefEdit' check and an 'MS or PS' check.
; 20190414 : No longer using the BKG_AutoMask_IsMtext_P and BKG_AutoMask_HasMask_P functions.
; 20190413
(defun BKG_AutoMask_CallBackObjAppend (rea lst)
  ; (print "> BKG_AutoMask_CallBackObjAppend")
  (if
    (and
      (vl-position (getvar 'cmdnames) '("FIELD" "MTEXT"))
      (= "MTEXT" (vle-entget 0 (cadr lst))) ; Required!
    )
    (progn
      ; (print "> BKG_AutoMask_CallBackObjAppend > Add to list")
      (setq *BKG_AutoMask_toDoLst* (cons (cadr lst) *BKG_AutoMask_toDoLst*))
    )
  )
)

; 20190413
(defun BKG_AutoMask_CallBackCmdOrLspStart (rea lst)
  (setq *BKG_AutoMask_cmdAndLspCount* (1+ *BKG_AutoMask_cmdAndLspCount*))
)

; 20190413
(defun BKG_AutoMask_CallBackCmdOrLspEnd (rea lst)
  (if (< 0 *BKG_AutoMask_cmdAndLspCount*)
    (setq *BKG_AutoMask_cmdAndLspCount* (1- *BKG_AutoMask_cmdAndLspCount*))
  )
  (if
    (and
      (zerop *BKG_AutoMask_cmdAndLspCount*)
      *BKG_AutoMask_toDoLst*
    )
    (BKG_AutoMask_Process)
  )
)

; 20190414: Added 'erased' check.
; 20190413
(defun BKG_AutoMask_Process ( / doneLst)
  ; (print "> BKG_AutoMask_Process")
  ; (print *BKG_AutoMask_toDoLst*)
  (setq
    *BKG_AutoMask_toDoLst*
    (vl-remove
      nil
      (mapcar
        '(lambda (enm)
          (cond
            ((vl-position enm doneLst)
              nil
            )
            ((vlax-erased-p enm)
              nil
            )
            ((not (vlax-write-enabled-p enm))
              enm
            )
            (T
              ; (print "> BKG_AutoMask_Process > Change")
              (BKG_AutoMask_AddMask enm *BKG_AutoMask_color* *BKG_AutoMask_offsetFactor*)
              (setq doneLst (cons enm doneLst))
              nil
            )
          )
        )
        *BKG_AutoMask_toDoLst*
      )
    )
  )
  (princ)
)

; 20190413
(defun c:BKG_AutoMask_Start ()
  ;; Remove existing reactors:
  (mapcar
    '(lambda (rea)
      (if (= "BKG_AutoMask" (vlr-data rea))
        (vlr-remove rea)
      )
    )
    (apply 'append (mapcar 'cdr (vlr-reactors)))
  )
  (vlr-acdb-reactor
    "BKG_AutoMask"
    '((:vlr-objectappended . BKG_AutoMask_CallBackObjAppend))
  )
  (vlr-command-reactor
    "BKG_AutoMask"
    '(
      (:vlr-commandcancelled . BKG_AutoMask_CallBackCmdOrLspEnd)
      (:vlr-commandended . BKG_AutoMask_CallBackCmdOrLspEnd)
      (:vlr-commandfailed . BKG_AutoMask_CallBackCmdOrLspEnd)
      (:vlr-commandwillstart . BKG_AutoMask_CallBackCmdOrLspStart)
    )
  )
  (vlr-lisp-reactor
    "BKG_AutoMask"
    '(
      (:vlr-lispcancelled . BKG_AutoMask_CallBackCmdOrLspEnd)
      (:vlr-lispended . BKG_AutoMask_CallBackCmdOrLspEnd)
      (:vlr-lispwillstart . BKG_AutoMask_CallBackCmdOrLspStart)
    )
  )
  (setq *BKG_AutoMask_cmdAndLspCount* 0)
  (setq *BKG_AutoMask_toDoLst* nil)
  (princ)
)

; 20190413
(defun c:BKG_AutoMask_Stop ()
  ;; Remove existing reactors:
  (mapcar
    '(lambda (rea)
      (if (= "BKG_AutoMask" (vlr-data rea))
        (vlr-remove rea)
      )
    )
    (apply 'append (mapcar 'cdr (vlr-reactors)))
  )
  (princ)
)

(c:BKG_AutoMask_Start)
(princ "\nBKG_AutoMask loaded ")
(princ)
